@@ -51,14 +51,10 @@ _DECLARE_XSTRTOL (xstrtoll, long long int)
 _DECLARE_XSTRTOL (xstrtoull, unsigned long long int)
 #endif
 
-#ifndef __attribute__
-# if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 8)
-#  define __attribute__(x)
-# endif
-#endif
-
-#ifndef ATTRIBUTE_NORETURN
-# define ATTRIBUTE_NORETURN __attribute__ ((__noreturn__))
+#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 8)
+# define _GL_ATTRIBUTE_NORETURN __attribute__ ((__noreturn__))
+#else
+# define _GL_ATTRIBUTE_NORETURN /* empty */
 #endif
 
 /* Report an error for an invalid integer in an option argument.
@@ -78,6 +74,6 @@ _DECLARE_XSTRTOL (xstrtoull, unsigned long long int)
 
 void xstrtol_fatal (enum strtol_error,
                     int, char, struct option const *,
-                    char const *) ATTRIBUTE_NORETURN;
+                    char const *) _GL_ATTRIBUTE_NORETURN;
 
 #endif /* not XSTRTOL_H_ */
