@@ -2,7 +2,7 @@ static char const G_line[] =
 {
   '#',' ','C','o','n','f','i','g','u','r','a','t','i','o','n',' ','f','i','l','e',' ','f','o','r',' ','d','i','r','c','o','l','o','r','s',',',' ','a',' ','u','t','i','l','i','t','y',' ','t','o',' ','h','e','l','p',' ','y','o','u',' ','s','e','t',' ','t','h','e',0,
   '#',' ','L','S','_','C','O','L','O','R','S',' ','e','n','v','i','r','o','n','m','e','n','t',' ','v','a','r','i','a','b','l','e',' ','u','s','e','d',' ','b','y',' ','G','N','U',' ','l','s',' ','w','i','t','h',' ','t','h','e',' ','-','-','c','o','l','o','r',' ','o','p','t','i','o','n','.',0,
-  '#',' ','C','o','p','y','r','i','g','h','t',' ','(','C',')',' ','1','9','9','6',',',' ','1','9','9','9','-','2','0','1','0',' ','F','r','e','e',' ','S','o','f','t','w','a','r','e',' ','F','o','u','n','d','a','t','i','o','n',',',' ','I','n','c','.',0,
+  '#',' ','C','o','p','y','r','i','g','h','t',' ','(','C',')',' ','1','9','9','6',',',' ','1','9','9','9','-','2','0','1','1',' ','F','r','e','e',' ','S','o','f','t','w','a','r','e',' ','F','o','u','n','d','a','t','i','o','n',',',' ','I','n','c','.',0,
   '#',' ','C','o','p','y','i','n','g',' ','a','n','d',' ','d','i','s','t','r','i','b','u','t','i','o','n',' ','o','f',' ','t','h','i','s',' ','f','i','l','e',',',' ','w','i','t','h',' ','o','r',' ','w','i','t','h','o','u','t',' ','m','o','d','i','f','i','c','a','t','i','o','n',',',0,
   '#',' ','a','r','e',' ','p','e','r','m','i','t','t','e','d',' ','p','r','o','v','i','d','e','d',' ','t','h','e',' ','c','o','p','y','r','i','g','h','t',' ','n','o','t','i','c','e',' ','a','n','d',' ','t','h','i','s',' ','n','o','t','i','c','e',' ','a','r','e',' ','p','r','e','s','e','r','v','e','d','.',0,
   '#',' ','T','h','e',' ','k','e','y','w','o','r','d','s',' ','C','O','L','O','R',',',' ','O','P','T','I','O','N','S',',',' ','a','n','d',' ','E','I','G','H','T','B','I','T',' ','(','h','o','n','o','r','e','d',' ','b','y',' ','t','h','e',0,
@@ -37,15 +37,20 @@ static char const G_line[] =
   'T','E','R','M',' ','m','l','t','e','r','m',0,
   'T','E','R','M',' ','p','u','t','t','y',0,
   'T','E','R','M',' ','r','x','v','t',0,
+  'T','E','R','M',' ','r','x','v','t','-','2','5','6','c','o','l','o','r',0,
   'T','E','R','M',' ','r','x','v','t','-','c','y','g','w','i','n',0,
   'T','E','R','M',' ','r','x','v','t','-','c','y','g','w','i','n','-','n','a','t','i','v','e',0,
   'T','E','R','M',' ','r','x','v','t','-','u','n','i','c','o','d','e',0,
+  'T','E','R','M',' ','r','x','v','t','-','u','n','i','c','o','d','e','-','2','5','6','c','o','l','o','r',0,
+  'T','E','R','M',' ','r','x','v','t','-','u','n','i','c','o','d','e','2','5','6',0,
   'T','E','R','M',' ','s','c','r','e','e','n',0,
   'T','E','R','M',' ','s','c','r','e','e','n','-','2','5','6','c','o','l','o','r',0,
   'T','E','R','M',' ','s','c','r','e','e','n','-','2','5','6','c','o','l','o','r','-','b','c','e',0,
   'T','E','R','M',' ','s','c','r','e','e','n','-','b','c','e',0,
   'T','E','R','M',' ','s','c','r','e','e','n','-','w',0,
+  'T','E','R','M',' ','s','c','r','e','e','n','.','r','x','v','t',0,
   'T','E','R','M',' ','s','c','r','e','e','n','.','l','i','n','u','x',0,
+  'T','E','R','M',' ','t','e','r','m','i','n','a','t','o','r',0,
   'T','E','R','M',' ','v','t','1','0','0',0,
   'T','E','R','M',' ','x','t','e','r','m',0,
   'T','E','R','M',' ','x','t','e','r','m','-','1','6','c','o','l','o','r',0,
@@ -119,6 +124,9 @@ static char const G_line[] =
   '.','d','e','b',' ','0','1',';','3','1',0,
   '.','r','p','m',' ','0','1',';','3','1',0,
   '.','j','a','r',' ','0','1',';','3','1',0,
+  '.','w','a','r',' ','0','1',';','3','1',0,
+  '.','e','a','r',' ','0','1',';','3','1',0,
+  '.','s','a','r',' ','0','1',';','3','1',0,
   '.','r','a','r',' ','0','1',';','3','1',0,
   '.','a','c','e',' ','0','1',';','3','1',0,
   '.','z','o','o',' ','0','1',';','3','1',0,
